'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.FailedBuildError = exports.LIFECYCLE_SCRIPTS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.build = build;
exports.parseLifecycleScripts = parseLifecycleScripts;
exports.buildAll = buildAll;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ArrayObservable = require('rxjs/observable/ArrayObservable');

var _Observable = require('rxjs/Observable');

var _do2 = require('rxjs/operator/do');

var _map = require('rxjs/operator/map');

var _concatMap = require('rxjs/operator/concatMap');

var _mergeMap = require('rxjs/operator/mergeMap');

var _filter = require('rxjs/operator/filter');

var _every = require('rxjs/operator/every');

var _child_process = require('child_process');

var _config = require('./config');

var config = _interopRequireWildcard(_config);

var _debuglog = require('./debuglog');

var _debuglog2 = _interopRequireDefault(_debuglog);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var log = (0, _debuglog2.default)('build');

/**
 * names of lifecycle scripts that should be run as part of the installation
 * process of a specific package (= properties of `scripts` object in
 * `package.json`).
 * @type {Array.<String>}
 * @readonly
 */
var LIFECYCLE_SCRIPTS = exports.LIFECYCLE_SCRIPTS = ['preinstall', 'install', 'postinstall'];

/**
 * error class used for representing an error that occurs due to a lifecycle
 * script that exits with a non-zero status code.
 */

var FailedBuildError = exports.FailedBuildError = function (_Error) {
	_inherits(FailedBuildError, _Error);

	/**
  * create instance.
  */
	function FailedBuildError() {
		_classCallCheck(this, FailedBuildError);

		var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(FailedBuildError).call(this, 'failed to build one or more dependencies that exited with != 0'));

		_this.name = FailedBuildError;
		return _this;
	}

	return FailedBuildError;
}(Error);

/**
 * build a dependency by executing the given lifecycle script.
 * @param  {String} nodeModules - absolute path of the `node_modules` directory.
 * @param  {Object} dep - dependency to be built.
 * @param  {String} dep.target - relative location of the target directory.
 * @param  {String} dep.script - script to be executed (usually using `sh`).
 * @return {Observable} - observable sequence of the returned exit code.
 */


function build(nodeModules, dep) {
	var target = dep.target;
	var script = dep.script;

	log('executing "' + script + '" from ' + target);

	return _Observable.Observable.create(function (observer) {
		// some packages do expect a defined `npm_execpath` env
		// eg. https://github.com/chrisa/node-dtrace-provider/blob/v0.6.0/scripts/install.js#L19
		var env = _extends({ npm_execpath: '' }, process.env);

		env.PATH = [_path2.default.join(nodeModules, target, 'node_modules', '.bin'), _path2.default.resolve(__dirname, '..', 'node_modules', '.bin'), process.env.PATH].join(_path2.default.delimiter);

		var childProcess = (0, _child_process.spawn)(config.sh, [config.shFlag, script], {
			cwd: _path2.default.join(nodeModules, target, 'package'),
			env: env,
			stdio: 'inherit'
			// shell: true // does break `dtrace-provider@0.6.0` build
		});
		childProcess.on('error', function (error) {
			observer.error(error);
		});
		childProcess.on('close', function (code) {
			observer.next(code);
			observer.complete();
		});
	});
}

/**
 * extract lifecycle scripts from supplied dependency.
 * @param {Dep} dep - dependency to be parsed.
 * @return {Array.<Object>} - array of script targets to be executed.
 */
function parseLifecycleScripts(_ref) {
	var target = _ref.target;
	var _ref$pkgJson$scripts = _ref.pkgJson.scripts;
	var scripts = _ref$pkgJson$scripts === undefined ? {} : _ref$pkgJson$scripts;

	var results = [];
	for (var i = 0; i < LIFECYCLE_SCRIPTS.length; i++) {
		var name = LIFECYCLE_SCRIPTS[i];
		var script = scripts[name];
		if (script) results.push({ target: target, script: script });
	}
	return results;
}

/**
 * run all lifecycle scripts upon completion of the installation process.
 * ensures that all scripts exit with 0 (success), otherwise an error will be
 * thrown.
 * @param  {String} nodeModules - `node_modules` base directory.
 * @return {Observable} - empty observable sequence that will be completed once
 * all lifecycle scripts have been executed.
 */
function buildAll(nodeModules) {
	var _context;

	return (_context = (_context = (_context = (_context = (_context = _map.map.call(this, parseLifecycleScripts), _mergeMap.mergeMap).call(_context, function (scripts) {
		return _ArrayObservable.ArrayObservable.create(scripts);
	}), _concatMap.concatMap).call(_context, function (script) {
		return build(nodeModules, script);
	}), _every.every).call(_context, function (code) {
		return code === 0;
	}), _filter.filter).call(_context, function (ok) {
		return !ok;
	}), _do2._do).call(_context, function () {
		throw new FailedBuildError();
	});
}