'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.init = init;
exports.getTmp = getTmp;
exports.write = write;
exports.read = read;
exports.extract = extract;

var _Observable = require('rxjs/Observable');

var _mergeMap = require('rxjs/operator/mergeMap');

var _retryWhen = require('rxjs/operator/retryWhen');

var _gunzipMaybe = require('gunzip-maybe');

var _gunzipMaybe2 = _interopRequireDefault(_gunzipMaybe);

var _tarFs = require('tar-fs');

var _tarFs2 = _interopRequireDefault(_tarFs);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _util = require('./util');

var util = _interopRequireWildcard(_util);

var _config = require('./config');

var config = _interopRequireWildcard(_config);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * initialize the cache.
 * @return {Observable} - observable sequence that will be completed once the
 * base directory of the cache has been created.
 */
function init() {
	return util.mkdirp(_path2.default.join(config.cacheDir, '.tmp'));
}

/**
 * get a random temporary filename.
 * @return {String} - temporary filename.
 */
function getTmp() {
	var filename = _path2.default.join(config.cacheDir, '.tmp', _nodeUuid2.default.v4());
	return filename;
}

/**
 * open a write stream into a temporarily cached file for caching a new
 * package.
 * @return {WriteStream} - Write Stream
 */
function write() {
	return _fs2.default.createWriteStream(getTmp());
}

/**
 * open a read stream to a cached dependency.
 * @param  {String} id - id (unique identifier) of the cached tarball.
 * @return {ReadStream} - Read Stream
 */
function read(id) {
	var filename = _path2.default.join(config.cacheDir, id);
	return _fs2.default.createReadStream(filename);
}

/**
 * extract a dependency from the cache.
 * @param {String} dest - pathname into which the cached dependency should be
 * extracted.
 * @param {String} id - id (unique identifier) of the cached tarball.
 * @return {Observable} - observable sequence that will be completed once
 * the cached dependency has been fetched.
 */
function extract(dest, id) {
	var _context,
	    _this = this;

	return (_context = _Observable.Observable.create(function (observer) {
		var tmpDest = getTmp();
		var untar = _tarFs2.default.extract(tmpDest, { strip: 1 });

		var completeHandler = function completeHandler() {
			observer.next(tmpDest);
			observer.complete();
		};
		var errorHandler = function errorHandler(err) {
			return observer.error(err);
		};

		_this.read(id).on('error', errorHandler).pipe((0, _gunzipMaybe2.default)()).on('error', errorHandler).pipe(untar).on('error', errorHandler).on('finish', completeHandler);
	}), _mergeMap.mergeMap).call(_context, function (tmpDest) {
		var _context2;

		return (_context2 = util.rename(tmpDest, dest), _retryWhen.retryWhen).call(_context2, function (errors) {
			return _mergeMap.mergeMap.call(errors, function (error) {
				if (error.code !== 'ENOENT') {
					throw error;
				}
				return util.mkdirp(_path2.default.dirname(dest));
			});
		});
	});
}