'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.LocalConflictError = exports.DEPENDENCY_FIELDS = exports.ENTRY_DEPENDENCY_FIELDS = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.resolveLocal = resolveLocal;
exports.resolveRemote = resolveRemote;
exports.resolveFromNpm = resolveFromNpm;
exports.resolveFromTarball = resolveFromTarball;
exports.resolveFromHosted = resolveFromHosted;
exports.resolveFromGit = resolveFromGit;
exports.resolve = resolve;
exports.resolveAll = resolveAll;
exports.linkAll = linkAll;
exports.fetchAll = fetchAll;

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _ArrayObservable = require('rxjs/observable/ArrayObservable');

var _EmptyObservable = require('rxjs/observable/EmptyObservable');

var _Observable = require('rxjs/Observable');

var _finally2 = require('rxjs/operator/finally');

var _concat = require('rxjs/operator/concat');

var _distinctKey = require('rxjs/operator/distinctKey');

var _expand = require('rxjs/operator/expand');

var _map = require('rxjs/operator/map');

var _catch2 = require('rxjs/operator/catch');

var _mergeMap = require('rxjs/operator/mergeMap');

var _retry = require('rxjs/operator/retry');

var _skip = require('rxjs/operator/skip');

var _semver = require('semver');

var _needle = require('needle');

var _needle2 = _interopRequireDefault(_needle);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _npmPackageArg = require('npm-package-arg');

var _npmPackageArg2 = _interopRequireDefault(_npmPackageArg);

var _lodash = require('lodash.memoize');

var _lodash2 = _interopRequireDefault(_lodash);

var _cache = require('./cache');

var cache = _interopRequireWildcard(_cache);

var _config = require('./config');

var config = _interopRequireWildcard(_config);

var _registry = require('./registry');

var registry = _interopRequireWildcard(_registry);

var _git = require('./git');

var git = _interopRequireWildcard(_git);

var _util = require('./util');

var util = _interopRequireWildcard(_util);

var _progress = require('./progress');

var progress = _interopRequireWildcard(_progress);

var _pkg_json = require('./pkg_json');

var _debuglog = require('./debuglog');

var _debuglog2 = _interopRequireDefault(_debuglog);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var log = (0, _debuglog2.default)('install');
var cachedNpa = (0, _lodash2.default)(_npmPackageArg2.default);

/**
 * properties of project-level `package.json` files that will be checked for
 * dependencies.
 * @type {Array.<String>}
 * @readonly
 */
var ENTRY_DEPENDENCY_FIELDS = exports.ENTRY_DEPENDENCY_FIELDS = ['dependencies', 'devDependencies', 'optionalDependencies'];

/**
 * properties of `package.json` of sub-dependencies that will be checked for
 * dependences.
 * @type {Array.<String>}
 * @readonly
 */
var DEPENDENCY_FIELDS = exports.DEPENDENCY_FIELDS = ['dependencies', 'optionalDependencies'];

/**
 * error class used for representing an error that occurs due to a lifecycle
 * script that exits with a non-zero status code.
 */

var LocalConflictError = exports.LocalConflictError = function (_Error) {
	_inherits(LocalConflictError, _Error);

	/**
  * create instance.
 	 * @param	{String} name - name of the dependency.
 	 * @param	{String} version - local version.
 	 * @param	{String} expected - expected version.
  */
	function LocalConflictError(name, version, expected) {
		_classCallCheck(this, LocalConflictError);

		var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(LocalConflictError).call(this, 'Local version ' + name + '@' + version + ' does not match required version @' + expected));

		_this.name = 'LocalConflictError';
		return _this;
	}

	return LocalConflictError;
}(Error);

/**
 * resolve a dependency's `package.json` file from the local file system.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{String} name - name of the dependency.
 * @param	{String} version - version of the dependency.
 * @param	{Boolean} isExplicit - whether the install command asks for an explicit install.
 * @return {Observable} - observable sequence of `package.json` objects.
 */


function resolveLocal(nodeModules, parentTarget, name, version, isExplicit) {
	var _context2;

	var linkname = _path2.default.join(nodeModules, parentTarget, 'node_modules', name);
	var mockFetch = function mockFetch() {
		return _EmptyObservable.EmptyObservable.create();
	};
	log('resolving ' + linkname + ' from node_modules');

	// support `file:` with symlinks
	if (version.substr(0, 5) === 'file:') {
		var _context;

		log('resolved ' + name + '@' + version + ' as local symlink');
		var isScoped = name.charAt(0) === '@';
		var src = _path2.default.join(parentTarget, isScoped ? '..' : '', version.substr(5));
		var dst = _path2.default.join('node_modules', parentTarget, 'node_modules', name);
		return (_context = util.forceSymlink(src, dst), _finally2._finally).call(_context, progress.complete);
	}

	return (_context2 = util.readlink(linkname), _mergeMap.mergeMap).call(_context2, function (rel) {
		var _context3;

		var target = _path2.default.basename(_path2.default.dirname(rel));
		var filename = _path2.default.join(linkname, 'package.json');
		log('reading package.json from ' + filename);

		return (_context3 = util.readFileJSON(filename), _map.map).call(_context3, function (pkgJson) {
			if (isExplicit && !(0, _semver.satisfies)(pkgJson.version, version)) {
				throw new LocalConflictError(name, pkgJson.version, version);
			}
			return { parentTarget: parentTarget, pkgJson: pkgJson, target: target, name: name, fetch: mockFetch };
		});
	});
}

/**
 * resolve a dependency's `package.json` file from a remote registry.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{String} name - name of the dependency.
 * @param	{String} version - version of the dependency.
 * @param	{Boolean} isExplicit - whether the install command asks for an explicit install.
 * @return {Observable} - observable sequence of `package.json` objects.
 */
function resolveRemote(nodeModules, parentTarget, name, version, isExplicit) {
	var source = name + '@' + version;
	log('resolving ' + source + ' from remote registry');

	var parsedSpec = cachedNpa(source);

	switch (parsedSpec.type) {
		case 'range':
		case 'version':
		case 'tag':
			return resolveFromNpm(nodeModules, parentTarget, parsedSpec);
		case 'remote':
			return resolveFromTarball(nodeModules, parentTarget, parsedSpec);
		case 'hosted':
			return resolveFromHosted(nodeModules, parentTarget, parsedSpec);
		case 'git':
			return resolveFromGit(nodeModules, parentTarget, parsedSpec);
		default:
			throw new Error('Unknown package spec: ' + parsedSpec.type + ' for ' + name);
	}
}

/**
 * resolve a dependency's `package.json` file from the npm registry.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{Object} parsedSpec - parsed package name and specifier.
 * @return {Observable} - observable sequence of `package.json` objects.
 */
function resolveFromNpm(nodeModules, parentTarget, parsedSpec) {
	var _context4;

	var raw = parsedSpec.raw;
	var name = parsedSpec.name;
	var type = parsedSpec.type;
	var spec = parsedSpec.spec;

	log('resolving ' + raw + ' from npm');
	var options = _extends({}, config.httpOptions, { retries: config.retries });
	return (_context4 = registry.match(name, spec, options), _map.map).call(_context4, function (pkgJson) {
		var target = pkgJson.dist.shasum;
		log('resolved ' + raw + ' to tarball shasum ' + target + ' from npm');
		return { parentTarget: parentTarget, pkgJson: pkgJson, target: target, name: name, type: type, fetch: fetch };
	});
}

/**
 * resolve a dependency's `package.json` file from an url tarball.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{Object} parsedSpec - parsed package name and specifier.
 * @return {Observable} - observable sequence of `package.json` objects.
 */
function resolveFromTarball(nodeModules, parentTarget, parsedSpec) {
	var raw = parsedSpec.raw;
	var name = parsedSpec.name;
	var type = parsedSpec.type;
	var spec = parsedSpec.spec;

	log('resolving ' + raw + ' from tarball');
	return _Observable.Observable.create(function (observer) {
		// create shasum from url for storage
		var hash = _crypto2.default.createHash('sha1');
		hash.update(raw);
		var shasum = hash.digest('hex');
		var pkgJson = { name: name, dist: { tarball: spec, shasum: shasum } };
		log('resolved ' + raw + ' to uri shasum ' + shasum + ' from tarball');
		observer.next({ parentTarget: parentTarget, pkgJson: pkgJson, target: shasum, name: name, type: type, fetch: fetch });
		observer.complete();
	});
}

/**
 * resolve a dependency's `package.json` file from an hosted GitHub-like registry.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{Object} parsedSpec - parsed package name and specifier.
 * @return {Observable} - observable sequence of `package.json` objects.
 */
function resolveFromHosted(nodeModules, parentTarget, parsedSpec) {
	var _context5;

	var raw = parsedSpec.raw;
	var name = parsedSpec.name;
	var type = parsedSpec.type;
	var hosted = parsedSpec.hosted;

	log('resolving ' + raw + ' from ' + hosted.type);

	var _hosted$shortcut$spli = hosted.shortcut.split(':');

	var _hosted$shortcut$spli2 = _slicedToArray(_hosted$shortcut$spli, 2);

	var provider = _hosted$shortcut$spli2[0];
	var shortcut = _hosted$shortcut$spli2[1];

	var _shortcut$split = shortcut.split('#');

	var _shortcut$split2 = _slicedToArray(_shortcut$split, 2);

	var repo = _shortcut$split2[0];
	var _shortcut$split2$ = _shortcut$split2[1];
	var ref = _shortcut$split2$ === undefined ? 'master' : _shortcut$split2$;


	var options = _extends({}, config.httpOptions, { retries: config.retries });
	// create shasum from directUrl for storage
	var hash = _crypto2.default.createHash('sha1');
	hash.update(hosted.directUrl);
	var shasum = hash.digest('hex');

	var tarball = void 0;
	switch (hosted.type) {
		case 'github':
			tarball = _url2.default.resolve('https://codeload.github.com', repo + '/tar.gz/' + ref);
			break;
		case 'bitbucket':
			tarball = _url2.default.resolve('https://bitbucket.org', repo + '/get/' + ref + '.tar.gz');
			break;
		default:
			throw new Error('Unknown hosted type: ' + hosted.type + ' for ' + name);
	}

	return (_context5 = (_context5 = registry.fetch(hosted.directUrl, options), _map.map).call(_context5, function (_ref) {
		var body = _ref.body;
		return JSON.parse(body);
	}), _map.map).call(_context5, function (pkgJson) {
		pkgJson.dist = { tarball: tarball, shasum: shasum }; // eslint-disable-line no-param-reassign
		log('resolved ' + name + '@' + ref + ' to directUrl shasum ' + shasum + ' from ' + provider);
		return { parentTarget: parentTarget, pkgJson: pkgJson, target: shasum, name: pkgJson.name, type: type, fetch: fetch };
	});
}

/**
 * resolve a dependency's `package.json` file from a git endpoint.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - relative parent's node_modules path.
 * @param	{Object} parsedSpec - parsed package name and specifier.
 * @return {Observable} - observable sequence of `package.json` objects.
 */
function resolveFromGit(nodeModules, parentTarget, parsedSpec) {
	var _context6;

	var raw = parsedSpec.raw;
	var type = parsedSpec.type;
	var spec = parsedSpec.spec;

	var mockFetch = function mockFetch() {
		return _EmptyObservable.EmptyObservable.create();
	};
	log('resolving ' + raw + ' from git');

	var _spec$split = spec.split('://');

	var _spec$split2 = _slicedToArray(_spec$split, 2);

	var protocol = _spec$split2[0];
	var host = _spec$split2[1];

	var _host$split = host.split('#');

	var _host$split2 = _slicedToArray(_host$split, 2);

	var repo = _host$split2[0];
	var _host$split2$ = _host$split2[1];
	var ref = _host$split2$ === undefined ? 'master' : _host$split2$;

	// create shasum from spec for storage

	var hash = _crypto2.default.createHash('sha1');
	hash.update(spec);
	var shasum = hash.digest('hex');
	var repoPath = void 0;

	return (_context6 = (_context6 = git.clone(repo, ref), _mergeMap.mergeMap).call(_context6, function (tmpDest) {
		repoPath = tmpDest;
		return util.readFileJSON(_path2.default.resolve(tmpDest, 'package.json'));
	}), _map.map).call(_context6, function (pkgJson) {
		var name = pkgJson.name;
		pkgJson.dist = { shasum: shasum, path: repoPath }; // eslint-disable-line no-param-reassign
		log('resolved ' + name + '@' + ref + ' to spec shasum ' + shasum + ' from git');
		return { parentTarget: parentTarget, pkgJson: pkgJson, target: shasum, name: name, type: type, fetch: git.extract };
	});
}

/**
 * resolve an individual sub-dependency based on the parent's target and the
 * current working directory.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{String} parentTarget - target path used for determining the sub-
 * dependency's path.
 * @param	{Boolean} isExplicit - whether the install command asks for an explicit install.
 * @return {Obserable} - observable sequence of `package.json` root documents
 * wrapped into dependency objects representing the resolved sub-dependency.
 */
function resolve(nodeModules, parentTarget, isExplicit) {
	return _mergeMap.mergeMap.call(this, function (_ref2) {
		var _context7;

		var _ref3 = _slicedToArray(_ref2, 2);

		var name = _ref3[0];
		var version = _ref3[1];

		progress.add();
		progress.report('resolving ' + name + '@' + version);
		log('resolving ' + name + '@' + version);

		return (_context7 = (_context7 = resolveLocal(nodeModules, parentTarget, name, version, isExplicit), _catch2._catch).call(_context7, function (error) {
			if (error.name !== 'LocalConflictError' && error.code !== 'ENOENT') {
				throw error;
			}
			log('failed to resolve ' + name + '@' + version + ' from local ' + parentTarget + ' via ' + nodeModules);
			return resolveRemote(nodeModules, parentTarget, name, version, isExplicit);
		}), _finally2._finally).call(_context7, progress.complete);
	});
}

/**
 * resolve all dependencies starting at the current working directory.
 * @param	{String} nodeModules - `node_modules` base directory.
 * @param	{Object} [targets=Object.create(null)] - resolved / active targets.
 * @param	{Boolean} isExplicit - whether the install command asks for an explicit install.
 * @return {Observable} - an observable sequence of resolved dependencies.
 */
function resolveAll(nodeModules) {
	var targets = arguments.length <= 1 || arguments[1] === undefined ? Object.create(null) : arguments[1];
	var isExplicit = arguments[2];

	return _expand.expand.call(this, function (_ref4) {
		var _context8;

		var target = _ref4.target;
		var pkgJson = _ref4.pkgJson;
		var _ref4$isProd = _ref4.isProd;
		var isProd = _ref4$isProd === undefined ? false : _ref4$isProd;

		// cancel when we get into a circular dependency
		if (target in targets) {
			log('aborting due to circular dependency ' + target);
			return _EmptyObservable.EmptyObservable.create();
		}

		targets[target] = true; // eslint-disable-line no-param-reassign

		// install devDependencies of entry dependency (project-level)
		var fields = target === '..' && !isProd ? ENTRY_DEPENDENCY_FIELDS : DEPENDENCY_FIELDS;

		log('extracting ' + fields + ' from ' + target);

		var dependencies = (0, _pkg_json.parseDependencies)(pkgJson, fields);

		return (_context8 = _ArrayObservable.ArrayObservable.create(dependencies), resolve).call(_context8, nodeModules, target, isExplicit);
	});
}

function resolveSymlink(src, dst) {
	var relSrc = _path2.default.relative(_path2.default.dirname(dst), src);
	return [relSrc, dst];
}

function getBinLinks(dep) {
	var pkgJson = dep.pkgJson;
	var parentTarget = dep.parentTarget;
	var target = dep.target;

	var binLinks = [];
	var bin = (0, _pkg_json.normalizeBin)(pkgJson);
	var names = Object.keys(bin);
	for (var i = 0; i < names.length; i++) {
		var name = names[i];
		var src = _path2.default.join('node_modules', target, 'package', bin[name]);
		var dst = _path2.default.join('node_modules', parentTarget, 'node_modules', '.bin', name);
		binLinks.push([src, dst]);
	}
	return binLinks;
}

function getDirectLink(dep) {
	var parentTarget = dep.parentTarget;
	var target = dep.target;
	var name = dep.name;

	var src = _path2.default.join('node_modules', target, 'package');
	var dst = _path2.default.join('node_modules', parentTarget, 'node_modules', name);
	return [src, dst];
}

/**
 * symlink the intermediate results of the underlying observable sequence
 * @param	{String} nodeModules - `node_modules` base directory.
 * @return {Observable} - empty observable sequence that will be completed
 * once all dependencies have been symlinked.
 */
function linkAll(nodeModules) {
	var _context9;

	return (_context9 = (_context9 = _mergeMap.mergeMap.call(this, function (dep) {
		return [getDirectLink(dep)].concat(_toConsumableArray(getBinLinks(dep)));
	}), _map.map).call(_context9, function (_ref5) {
		var _ref6 = _slicedToArray(_ref5, 2);

		var src = _ref6[0];
		var dst = _ref6[1];
		return resolveSymlink(src, dst);
	}), _mergeMap.mergeMap).call(_context9, function (_ref7) {
		var _ref8 = _slicedToArray(_ref7, 2);

		var src = _ref8[0];
		var dst = _ref8[1];

		log('symlinking ' + src + ' -> ' + dst);
		return util.forceSymlink(src, dst);
	});
}

function checkShasum(shasum, expected, tarball) {
	_assert2.default.equal(shasum, expected, 'shasum mismatch for ' + tarball + ': ' + shasum + ' <-> ' + expected);
}

function download(tarball, expected, type) {
	var _context10;

	log('downloading ' + tarball + ', expecting ' + expected);
	return (_context10 = _Observable.Observable.create(function (observer) {
		var shasum = _crypto2.default.createHash('sha1');
		var response = _needle2.default.get(tarball, config.httpOptions);
		var cached = response.pipe(cache.write());

		var errorHandler = function errorHandler(error) {
			return observer.error(error);
		};
		var dataHandler = function dataHandler(chunk) {
			return shasum.update(chunk);
		};
		var finishHandler = function finishHandler() {
			var actualShasum = shasum.digest('hex');
			log('downloaded ' + actualShasum + ' into ' + cached.path);
			// only actually check shasum integrity for npm tarballs
			var expectedShasum = ['range', 'version', 'tag'].indexOf(type) !== -1 ? actualShasum : expected;
			observer.next({ tmpPath: cached.path, shasum: expectedShasum });
			observer.complete();
		};

		response.on('data', dataHandler);
		response.on('error', errorHandler);

		cached.on('error', errorHandler);
		cached.on('finish', finishHandler);
	}), _mergeMap.mergeMap).call(_context10, function (_ref9) {
		var tmpPath = _ref9.tmpPath;
		var shasum = _ref9.shasum;

		if (expected) {
			checkShasum(shasum, expected, tarball);
		}

		var newPath = _path2.default.join(config.cacheDir, shasum);
		return util.rename(tmpPath, newPath);
	});
}

function fixPermissions(target, bin) {
	var _context11;

	var execMode = 511 & ~process.umask();
	var paths = [];
	var names = Object.keys(bin);
	for (var i = 0; i < names.length; i++) {
		var name = names[i];
		paths.push(_path2.default.resolve(target, bin[name]));
	}
	log('fixing persmissions of ' + names + ' in ' + target);
	return (_context11 = _ArrayObservable.ArrayObservable.create(paths), _mergeMap.mergeMap).call(_context11, function (filepath) {
		return util.chmod(filepath, execMode);
	});
}

function fetch(nodeModules) {
	var _context12;

	var target = this.target;
	var type = this.type;
	var _pkgJson = this.pkgJson;
	var name = _pkgJson.name;
	var bin = _pkgJson.bin;
	var _pkgJson$dist = _pkgJson.dist;
	var tarball = _pkgJson$dist.tarball;
	var shasum = _pkgJson$dist.shasum;

	var where = _path2.default.join(nodeModules, target, 'package');

	log('fetching ' + tarball + ' into ' + where);

	return (_context12 = (_context12 = util.stat(where), _skip.skip).call(_context12, 1), _catch2._catch).call(_context12, function (error) {
		var _context13;

		if (error.code !== 'ENOENT') {
			throw error;
		}
		var extracted = (_context13 = cache.extract(where, shasum), _catch2._catch).call(_context13, function (error) {
			// eslint-disable-line
			if (error.code !== 'ENOENT') {
				throw error;
			}
			return (0, _concat.concatStatic)(download(tarball, shasum, type), cache.extract(where, shasum));
		});
		var fixedPermissions = fixPermissions(where, (0, _pkg_json.normalizeBin)({ name: name, bin: bin }));
		return (0, _concat.concatStatic)(extracted, fixedPermissions);
	});
}

function fetchAll(nodeModules) {
	var _context15;

	var fetchWithRetry = function fetchWithRetry(dep) {
		var _context14;

		return (_context14 = dep.fetch(nodeModules), _retry.retry).call(_context14, config.retries);
	};
	return (_context15 = _distinctKey.distinctKey.call(this, 'target'), _mergeMap.mergeMap).call(_context15, fetchWithRetry);
}