'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.getSymlinks = getSymlinks;
exports.linkToGlobal = linkToGlobal;
exports.linkFromGlobal = linkFromGlobal;
exports.unlinkToGlobal = unlinkToGlobal;
exports.unlinkFromGlobal = unlinkFromGlobal;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _config = require('./config');

var config = _interopRequireWildcard(_config);

var _ArrayObservable = require('rxjs/observable/ArrayObservable');

var _util = require('./util');

var _mergeMap = require('rxjs/operator/mergeMap');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * generate the symlinks to be created in order to link to passed in package.
 * @param {String} cwd - current working directory.
 * @param {Object} pkgJson - `package.json` file to be linked.
 * @return {Array.<String>} - an array of tuples representing symbolic links to be
 * created.
 */
function getSymlinks(cwd, pkgJson) {
  var libSymlink = [cwd, _path2.default.join(config.globalNodeModules, pkgJson.name)];
  var bin = pkgJson.bin;
  if (typeof bin === 'string') {
    bin = {};
    bin[pkgJson.name] = pkgJson.bin;
  }
  bin = bin || {};
  var binSymlinks = Object.keys(bin).map(function (name) {
    return [_path2.default.join(config.globalNodeModules, pkgJson.name, bin[name]), _path2.default.join(config.globalBin, name)];
  });
  return [libSymlink].concat(binSymlinks);
}

/*
 * globally expose the package we're currently in (used for `ied link`).
 * @param {String} cwd - current working directory.
 * @return {Observable} - observable sequence.
 */
function linkToGlobal(cwd) {
  var _context;

  return (_context = (_context = (0, _util.readFileJSON)(_path2.default.join(cwd, 'package.json')), _mergeMap.mergeMap).call(_context, function (pkgJson) {
    return getSymlinks(cwd, pkgJson);
  }), _mergeMap.mergeMap).call(_context, function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2);

    var src = _ref2[0];
    var dst = _ref2[1];
    return (0, _util.forceSymlink)(src, dst);
  });
}

/**
 * links a globally linked package into the package present in the current
 * working directory (used for `ied link some-package`).
 * the package can be `require`d afterwards.
 * `node_modules/.bin` stays untouched.
 * @param {String} cwd - current working directory.
 * @param {String} name - name of the dependency to be linked.
 * @return {Observable} - observable sequence.
 */
function linkFromGlobal(cwd, name) {
  var dst = _path2.default.join(cwd, 'node_modules', name);
  var src = _path2.default.join(config.globalNodeModules, name);
  return (0, _util.forceSymlink)(src, dst);
}

/**
 * revert the effects of `ied link` by removing the previously created
 * symbolic links (used for `ied unlink`).
 * @param {String} cwd - current working directory.
 * @param {String} name - name of the dependency to be unlinked.
 * @return {Observable} - observable sequence.
 */
function unlinkToGlobal(cwd, name) {
  var _context2;

  var pkg = require(_path2.default.join(cwd, 'package.json'));
  var symlinks = getSymlinks(cwd, pkg);
  return (_context2 = _ArrayObservable.ArrayObservable.create(symlinks), _mergeMap.mergeMap).call(_context2, function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2);

    var src = _ref4[0];
    var dst = _ref4[1];
    return (0, _util.unlink)(dst);
  });
}

/**
 * revert the effects of `ied link some-package` by removing the previously
 * created symbolic links from the project's `node_modules` directory (used for
 * `ied unlink some-package`).
 * @param {String} cwd - current working directory.
 * @param {String} name - name of the dependency to be unlinked from the
 * project's `node_modules`.
 * @return {Observable} - observable sequence.
 */
function unlinkFromGlobal(cwd, name) {
  var dst = _path2.default.join(cwd, 'node_modules', name);
  return (0, _util.unlink)(dst);
}