'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = linkCmd;

var _config = require('./config');

var config = _interopRequireWildcard(_config);

var _util = require('./util');

var _link = require('./link');

var _concat = require('rxjs/operator/concat');

var _ArrayObservable = require('rxjs/observable/ArrayObservable');

var _mergeMap = require('rxjs/operator/mergeMap');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/**
 * can be used in two ways:
 * 1. in order to globally _expose_ the current package (`ied link`).
 * 2. in order to use a previously globally _exposed_ package (`ied link tap`).
 *
 * useful for local development when you want to use a dependency in a
 * different project without publishing to the npm registry / installing from
 * local FS.
 *
 * create a symlink either in the global `node_modules` directory (`ied link`)
 * or in the project's `node_modules` directory (e.g. `ied link tap` would
 * create a symlink in `current-project/node_modules/tap` pointing to a
 * globally installed tap version).
 *
 * @param  {String} cwd - current working directory.
 * @param  {Object} argv - parsed command line arguments.
 * @return {Observable} - observable sequence.
 */
function linkCmd(cwd, argv) {
  var names = argv._.slice(1);

  if (names.length) {
    var _context;

    var localNodeModules = _path2.default.join(cwd, 'node_modules');
    var _init = (0, _util.mkdirp)(localNodeModules);
    return (0, _concat.concatStatic)(_init, (_context = _ArrayObservable.ArrayObservable.create(names), _mergeMap.mergeMap).call(_context, function (name) {
      return (0, _link.linkFromGlobal)(cwd, name);
    }));
  }

  var init = (0, _concat.concatStatic)((0, _util.mkdirp)(config.globalNodeModules), (0, _util.mkdirp)(config.globalBin));
  return (0, _concat.concatStatic)(init, (0, _link.linkToGlobal)(cwd));
}