'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.add = add;
exports.complete = complete;
exports.report = report;

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var spinner = (0, _ora2.default)({
  spinner: 'clock'
});

var completed = 0;
var added = 0;
var status = '';

// start the spinner on startup
spinner.start();

/**
 * add one or more scheduled tasks.
 * @param {Number} [n=1] - number of scheduled tasks.
 */
function add() {
  var n = arguments.length <= 0 || arguments[0] === undefined ? 1 : arguments[0];

  added += n;
  report();
}

/**
 * complete a previously scheduled task. stop the spinner when there are no
 * outstanding tasks.
 * @param  {Number} [n=1] - number of tasks that have been completed.
 */
function complete() {
  var n = arguments.length <= 0 || arguments[0] === undefined ? 1 : arguments[0];

  completed += n;
  if (added === completed) spinner.stop();else report();
}

/**
 * log the progress by updating the status message, percentage and spinner.
 * @param  {String} [_status] - optional (updated) status message. defaults to
 * the previous status message.
 * @see https://www.npmjs.org/package/ora
 */
function report() {
  var _status = arguments.length <= 0 || arguments[0] === undefined ? status : arguments[0];

  status = _status;
  var progress = Math.round(completed / added * 100 * 100) / 100;
  spinner.text = progress + '% ' + status;
}