'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.requests = exports.DEFAULT_RETRIES = exports.DEFAULT_REGISTRY = undefined;
exports.reset = reset;
exports.checkStatus = checkStatus;
exports.escapeName = escapeName;
exports.fetch = fetch;
exports.match = match;

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _map = require('rxjs/operator/map');

var _do2 = require('rxjs/operator/do');

var _retry = require('rxjs/operator/retry');

var _publishReplay = require('rxjs/operator/publishReplay');

var _util = require('./util');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * default registry URL to be used. can be overridden via options on relevant
 * functions.
 * @type {String}
 */
var DEFAULT_REGISTRY = exports.DEFAULT_REGISTRY = 'https://registry.npmjs.org/';

/**
 * default number of retries to attempt before failing to resolve to a package
 * @type {Number}
 */
var DEFAULT_RETRIES = exports.DEFAULT_RETRIES = 5;

/**
 * register of pending and completed HTTP requests mapped to their respective
 * observable sequences.
 * @type {Object}
 */
var requests = exports.requests = Object.create(null);

/**
 * clear the internal cache used for pending and completed HTTP requests.
 */
function reset() {
  var uris = Object.keys(requests);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = uris[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var uri = _step.value;

      delete requests[uri];
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

/**
 * ensure that the registry responded with an accepted HTTP status code
 * (`200`).
 * @param  {String} uri - URI used for retrieving the supplied response.
 * @param  {Number} resp - HTTP response object.
 * @throws {assert.AssertionError} if the status code is not 200.
 */
function checkStatus(uri, resp) {
  var statusCode = resp.statusCode;
  var error = resp.body.error;

  _assert2.default.equal(statusCode, 200, 'error status code ' + uri + ': ' + error);
}

/**
 * escape the given package name, which can then be used as part of the package
 * root URL.
 * @param  {String} name - package name.
 * @return {String} - escaped package name.
 */
function escapeName(name) {
  var isScoped = name.charAt(0) === '@';
  var escapedName = isScoped ? '@' + encodeURIComponent(name.substr(1)) : encodeURIComponent(name);
  return escapedName;
}

/**
 * HTTP GET the resource at the supplied URI. if a request to the same URI has
 * already been made, return the cached (pending) request.
 * @param  {String} uri - endpoint to fetch data from.
 * @param  {Object} [options = {}] - optional HTTP and `retries` options.
 * @return {Observable} - observable sequence of pending / completed request.
 */
function fetch(uri) {
  var _context;

  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var _options$retries = options.retries;
  var retries = _options$retries === undefined ? DEFAULT_RETRIES : _options$retries;

  var needleOptions = _objectWithoutProperties(options, ['retries']);

  var existingRequest = requests[uri];

  if (existingRequest) {
    return existingRequest;
  }
  var newRequest = (_context = (_context = (_context = (0, _util.httpGet)(uri, needleOptions), _do2._do).call(_context, function (resp) {
    return checkStatus(uri, resp);
  }), _retry.retry).call(_context, retries), _publishReplay.publishReplay).call(_context).refCount();
  requests[uri] = newRequest;
  return newRequest;
}

/**
 * resolve a package defined via an ambiguous semantic version string to a
 * specific `package.json` file.
 * @param {String} name - package name.
 * @param {String} version - semantic version string or tag name.
 * @param {Object} options - HTTP request options.
 * @return {Observable} - observable sequence of the `package.json` file.
 */
function match(name, version) {
  var _context2;

  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var escapedName = escapeName(name);
  var _options$registry = options.registry;
  var registry = _options$registry === undefined ? DEFAULT_REGISTRY : _options$registry;

  var fetchOptions = _objectWithoutProperties(options, ['registry']);

  var uri = _url2.default.resolve(registry, escapedName + '/' + version);
  return (_context2 = fetch(uri, fetchOptions), _map.map).call(_context2, function (_ref) {
    var body = _ref.body;
    return body;
  });
}