'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = runCmd;
exports.run = run;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _util = require('./util');

var _concatMap = require('rxjs/operator/concatMap');

var _rxjs = require('rxjs');

var _map = require('rxjs/operator/map');

var _filter = require('rxjs/operator/filter');

var _reduce = require('rxjs/operator/reduce');

var _do2 = require('rxjs/operator/do');

var _child_process = require('child_process');

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * run a `package.json` script and the related pre- and postscript.
 * @param {String} cwd - current working directory.
 * @param  {Object} argv - command line arguments.
 * @return {Observable} - observable sequence.
 */
function runCmd(cwd, argv) {
	var _context;

	var scripts = argv._.slice(1);

	var pkgJson = (_context = (0, _util.readFile)(_path2.default.join(cwd, 'package.json'), 'utf8'), _map.map).call(_context, JSON.parse);

	// if no script(s) have been specified, log out the available scripts.
	if (!scripts.length) {
		return _do2._do.call(pkgJson, logAvailable);
	}

	var PATH = [_path2.default.join(cwd, 'node_modules/.bin'), process.env.PATH].join(_path2.default.delimiter);
	var env = _extends({}, process.env, { PATH: PATH });
	var runOptions = { env: env, stdio: 'inherit' };

	return (_context = (_context = (_context = (_context = (_context = (_context = _map.map.call(pkgJson, function (_ref) {
		var _ref$scripts = _ref.scripts;
		var scripts = _ref$scripts === undefined ? {} : _ref$scripts;
		return scripts;
	}), _util.entries).call(_context) // eslint-disable-line no-shadow
	, _filter.filter).call(_context, function (_ref2) {
		var _ref3 = _slicedToArray(_ref2, 1);

		var name = _ref3[0];
		return ~scripts.indexOf(name);
	}), _concatMap.concatMap).call(_context, function (_ref4) {
		var _context2;

		var _ref5 = _slicedToArray(_ref4, 2);

		var name = _ref5[0];
		var script = _ref5[1];
		return (_context2 = run(script, runOptions), _map.map).call(_context2, function (code) {
			return [name, script, code];
		});
	}), _do2._do).call(_context, logCode), _reduce.reduce).call(_context, function (codes, _ref6) {
		var _ref7 = _slicedToArray(_ref6, 3);

		var name = _ref7[0];
		var script = _ref7[1];
		var code = _ref7[2];
		return codes + code;
	}, 0), _do2._do).call(_context, function (code) {
		return _assert2.default.equal(code, 0, 'exit status != 0');
	});
}

function run(script) {
	var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

	return _rxjs.Observable.create(function (observer) {
		var args = [_config.shFlag, script];
		var childProcess = (0, _child_process.spawn)(_config.sh, args, options);
		childProcess.on('close', function (code) {
			observer.next(code);
			observer.complete();
		});
		childProcess.on('error', function (err) {
			observer.error(err);
		});
	});
}

function logCode(_ref8) {
	var _ref9 = _slicedToArray(_ref8, 3);

	var name = _ref9[0];
	var script = _ref9[1];
	var code = _ref9[2];

	var prefix = name + ': `' + script + '`';
	if (code === 0) console.log(prefix + ' succeeded');else console.error(prefix + ' failed (exit status ' + code + ')');
}

function logAvailable(_ref10) {
	var _ref10$scripts = _ref10.scripts;
	var scripts = _ref10$scripts === undefined ? {} : _ref10$scripts;

	var available = Object.keys(scripts);
	if (available.length) {
		console.log('available scripts: ' + available.join(', '));
	} else {
		console.log('no scripts in package.json');
	}
}