'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.mkdirp = exports.forceSymlink = exports.unlink = exports.chmod = exports.readdir = exports.readlink = exports.rename = exports.stat = exports.writeFile = exports.readFile = undefined;
exports.createObservableFactory = createObservableFactory;
exports.httpGet = httpGet;
exports.httpGetJSON = httpGetJSON;
exports.entries = entries;
exports.readFileJSON = readFileJSON;
exports.setTitle = setTitle;

var _Observable = require('rxjs/Observable');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp2 = require('mkdirp');

var _mkdirp3 = _interopRequireDefault(_mkdirp2);

var _forceSymlink2 = require('force-symlink');

var _forceSymlink3 = _interopRequireDefault(_forceSymlink2);

var _needle = require('needle');

var _needle2 = _interopRequireDefault(_needle);

var _map = require('rxjs/operator/map');

var _mergeMap = require('rxjs/operator/mergeMap');

var _config = require('./config');

var config = _interopRequireWildcard(_config);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * given an arbitrary asynchronous function that accepts a callback function,
 * wrap the outer asynchronous function into an observable sequence factory.
 * invoking the returned generated function is going to return a new **cold**
 * observable sequence.
 * @param  {Function} fn - function to be wrapped.
 * @param  {thisArg} [thisArg] - optional context.
 * @return {Function} - cold observable sequence factory.
 */
function createObservableFactory(fn, thisArg) {
	return function () {
		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _Observable.Observable.create(function (observer) {
			fn.apply(thisArg, [].concat(args, [function (error) {
				for (var _len2 = arguments.length, results = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
					results[_key2 - 1] = arguments[_key2];
				}

				if (error) {
					observer.error(error);
				} else {
					results.forEach(function (result) {
						return observer.next(result);
					});
					observer.complete();
				}
			}]));
		});
	};
}

/**
 * send a GET request to the given HTTP endpoint by passing the supplied
 * arguments to [`needle`](https://www.npmjs.com/package/needle).
 * @return {Observable} - observable sequence of a single response object.
 */
function httpGet() {
	for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
		args[_key3] = arguments[_key3];
	}

	return _Observable.Observable.create(function (observer) {
		_needle2.default.get.apply(_needle2.default, args.concat([function (error, response) {
			if (error) observer.error(error);else {
				observer.next(response);
				observer.complete();
			}
		}]));
	});
}

/**
 * GETs JSON documents from an HTTP endpoint.
 * @param  {String} url - endpoint to which the GET request should be made
 * @return {Object} An observable sequence of the fetched JSON document.
 */
function httpGetJSON(url) {
	return _Observable.Observable.create(function (observer) {
		_needle2.default.get(url, config.httpOptions, function (error, response) {
			if (error) observer.error(error);else {
				observer.next(response.body);
				observer.complete();
			}
		});
	});
}

/** @type {Function} Observable wrapper function around `fs.readFile`. */
var readFile = exports.readFile = createObservableFactory(_fs2.default.readFile, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.writeFile`. */
var writeFile = exports.writeFile = createObservableFactory(_fs2.default.writeFile, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.stat`. */
var stat = exports.stat = createObservableFactory(_fs2.default.stat, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.rename`. */
var rename = exports.rename = createObservableFactory(_fs2.default.rename, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.readlink`. */
var readlink = exports.readlink = createObservableFactory(_fs2.default.readlink, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.readdir`. */
var readdir = exports.readdir = createObservableFactory(_fs2.default.readdir, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.chmod`. */
var chmod = exports.chmod = createObservableFactory(_fs2.default.chmod, _fs2.default);

/** @type {Function} Observable wrapper function around `fs.unlink`. */
var unlink = exports.unlink = createObservableFactory(_fs2.default.unlink, _fs2.default);

/** @type {Function} Observable wrapper function around
[`force-symlink`](https://www.npmjs.org/package/force-symlink). */
var forceSymlink = exports.forceSymlink = createObservableFactory(_forceSymlink3.default);

/** @type {Function} Observable wrapper function around
[`mkdirp`](https://www.npmjs.com/package/mkdirp). */
var mkdirp = exports.mkdirp = createObservableFactory(_mkdirp3.default);

/**
 * equivalent to `Map#entries` for observables, but operates on objects.
 * @return {Observable} - observable sequence of pairs.
 */
function entries() {
	return _mergeMap.mergeMap.call(this, function (object) {
		var results = [];
		var keys = Object.keys(object);
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = keys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var key = _step.value;

				results.push([key, object[key]]);
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		return results;
	});
}

/**
 * read a UTF8 encoded JSON file from disk.
 * @param  {String} file - filename to be used.
 * @return {Observable} - observable sequence of a single object representing
 * the read JSON file.
 */
function readFileJSON(file) {
	var _context;

	return (_context = readFile(file, 'utf8'), _map.map).call(_context, JSON.parse);
}

/**
 * set the terminal title using the required ANSI escape codes.
 * @param {String} title - title to be set.
 */
function setTitle(title) {
	process.stdout.write(String.fromCharCode(27) + ']0;' + title + String.fromCharCode(7));
}